package com.zzys.lightting.filter;

import com.alibaba.fastjson.JSON;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.zzys.lightting.user.dao.model.UserInfo;
import com.zzys.lightting.user.service.UserService;
import com.zzys.lightting.utils.JWTUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;

/**
 * @author lpf
 * @description:
 * @date 2021/01/20 16:54
 */
@Slf4j
@Component
public class UserCheckInterceptor implements HandlerInterceptor {
    @Autowired
    private UserService userService;


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        HashMap<String, Object> resultJson = new HashMap<>();
        String uri = request.getRequestURI();
        //获取请求头token
        String token = request.getHeader("Authorization");
        String[] len = token.split(" ");
        if (len.length >1){
            token = len[1];
            JWTUtil.verify(token);
        } else if (len.length==1) {
            token = len[0];
            JWTUtil.verify(token);
        }
        //token = token.split(" ")[1];//以空格划分Bearer token，获取token
        //JWTUtil.verify(token);//验证令牌
        //从请求头中获取token
        DecodedJWT verify = JWTUtil.verify(token);

        String type = verify.getClaim("type").asString();
        String userId = verify.getClaim("userId").asString();
        boolean flag = true;

        UserInfo userInfo = this.userService.findById(userId);
//        UserInfo userInfo = this.userService.getOne(userId);
        if (userInfo == null) {
            flag = false;
            resultJson.put("code", HttpServletResponse.SC_UNAUTHORIZED);
            resultJson.put("msg", "账户已被删除，请联系管理员");
        }else {
            if(!userInfo.getId().equals(verify.getClaim("userId").asString())){
                resultJson.put("code", HttpServletResponse.SC_UNAUTHORIZED);
                resultJson.put("msg", "账户已被删除，请联系管理员");
                String s = JSON.toJSONString(resultJson);
                response.setContentType("application/json;charset=UTF-8");
                response.getWriter().println(s);
            }
        }


        if(!flag){
            resultJson.put("code",HttpServletResponse.SC_UNAUTHORIZED);
            resultJson.put("msg","未授权");
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            return false;
        }


        return true; //放行
    }
}
