package com.zzys.lightting.music.rest;

import com.alibaba.fastjson.JSONObject;
import com.zzys.lightting.base.rest.BaseController;
import com.zzys.lightting.music.dao.model.MusicInfo;
import com.zzys.lightting.music.dao.model.MusicType;
import com.zzys.lightting.music.service.MusicService;
import com.zzys.lightting.music.service.MusicTypeService;
import com.zzys.lightting.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author: zzf
 * @Date: 2024/5/23:14:41
 * @Description:
 */
@Slf4j
@RestController
@RequestMapping("/music")
@Api(tags = "音乐管理")
public class MusicController extends BaseController {


    @Autowired
    private MusicService service;

    @Autowired
    private MusicTypeService typeService;

    @PostMapping("/save")
    @ApiOperation(value = "保存-新增/修改",notes = "name:名称；author：作者；typeId：分类id；sourceFile：源文件")
    public Result save(@RequestBody String json){
        JSONObject jsonObject = JSONObject.parseObject(json);
        MusicInfo music = new MusicInfo();
        music.setName(jsonObject.getString("name"));
        music.setAuthor(jsonObject.getString("author"));
        music= this.service.save(music);
        MusicType type = new MusicType();
        type.setTypeId(jsonObject.getString("typeId"));
        type.setMusicId(music.getId());
        this.typeService.save(type);
        return success();
    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation("删除-id")
    public Result delete(@PathVariable String id){
        this.typeService.deleteByMusicId(id);
        this.service.deleteById(id);
        return success();
    }

    @GetMapping("/findAll")
    @ApiOperation("查询全部")
    public Result findAll(){
        List<MusicInfo> musicInfos = this.service.findAll();
        return success(musicInfos);
    }

    @GetMapping("/find/{typeId}/{pageNum}/{pageSize}")
    public Result findByType(@PathVariable String typeId, @PathVariable int pageNum, @PathVariable int pageSize){
        List<String> musicIdList = this.typeService.findMusicIdByTypeId(typeId);
        Page<MusicInfo> musicInfos = this.service.findByIds(musicIdList,pageNum,pageSize);
        return success(musicInfos);
    }

}
